import pytest
import xarray as xr
import numpy as np
import pandas as pd
from tinysunset.scalers.standard import StandardScaler
from tinysunset.scalers.base import ScalerConfig

@pytest.fixture
def sample_data():
    time = pd.date_range('2000-01-01', periods=100, freq='D')
    ensemble = np.arange(10)
    data = np.random.normal(loc=0.0, scale=1.0, size=(100, 10))
    da = xr.DataArray(data, coords=[time, ensemble], dims=['time', 'ensemble'])
    return da

@pytest.fixture
def scaler_config():
    return ScalerConfig(
        ensemble_dim='ensemble',
        time_dim='time',
        clim_type='clim',
        std_ref_years=[2000, 2001],
        clim_ref_years=[2000, 2001],
        loess_degree=2,
        loess_window=5,
        groupby_type=None,
        retro_window=10,
        alpha=0.5
    )

def test_fit(sample_data, scaler_config):
    scaler = StandardScaler(scaler_config)
    scaler.fit(sample_data)
    assert hasattr(scaler, 'mean')
    assert hasattr(scaler, 'std')
    np.testing.assert_allclose(scaler.mean, 0, atol=0.1)
    np.testing.assert_allclose(scaler.std, 1, atol=0.1)

def test_transform(sample_data, scaler_config):
    scaler = StandardScaler(scaler_config)
    scaler.fit(sample_data)
    transformed = scaler.transform(sample_data)
    assert isinstance(transformed, xr.DataArray)
    assert transformed.shape == sample_data.shape

def test_fit_transform(sample_data, scaler_config):
    scaler = StandardScaler(scaler_config)
    transformed = scaler.fit_transform(sample_data)
    assert isinstance(transformed, xr.DataArray)
    assert transformed.shape == sample_data.shape

def test_inverse_transform(sample_data, scaler_config):
    scaler = StandardScaler(scaler_config)
    transformed = scaler.fit_transform(sample_data)
    inverse_transformed = scaler.inverse_transform(transformed)
    assert isinstance(inverse_transformed, xr.DataArray)
    assert inverse_transformed.shape == sample_data.shape
    np.testing.assert_allclose(inverse_transformed, sample_data, rtol=1e-5)